package genplay.vp.game;
/*
 * File: Game.java
 * App : Optimal Video Poker
 * 
 * This Class handles all engine processing for the poker Games, including both
 * tracking the state of the current game (and providing the necessary strategy processing),
 * and keeping track of the various game statistics in general (as well their persistant storage).
 * 
 * This one class requires a two-stage precompile process: 
 * 
 * It is first run through the Antenna precompiler, which will remove all code below and 
 * Replace it with a more traditional set of #define (located in genplay/vp/core/bit.h),
 * and the second is a traditional precompile with CPP.  
 * 
 * The temporary code will indeed Execute, 
 * however, it is both slower and less space efficiant.
 * 
 */
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.IOException;

import genplay.vp.core.*;

import javax.microedition.io.*;
import java.io.*;
import java.util.Random;

import javax.microedition.lcdui.*;
import javax.microedition.rms.RecordStore;
import javax.microedition.rms.RecordStoreNotFoundException;

import genplay.util.*;
import genplay.vp.logi.*;

public class Game {
    
    /*
	 * --------------------- Main Static Defs --------------------------------------------------------
	 */

    // An application standard
    public static final int NONE = -1;
    
    // Card Spots
	public static final int NOSPOTS = -1;
	public static final int ACELOW = 0;								
	public static final int TWO = 1;
	public static final int THREE = 2;
	public static final int FOUR = 3;
	public static final int FIVE = 4;
	public static final int SIX = 5;
	public static final int SEVEN = 6;
	public static final int EIGHT = 7;
	public static final int NINE = 8;
	public static final int TEN = 9;
	public static final int JACK = 10;
	public static final int QUEEN = 11;
	public static final int KING = 12;
	public static final int ACE = 13;
	public static final int JOKER = 14;
	public static final int NUM_SPOTS = 15;
	public static final int WILD = NUM_SPOTS;
	
	// Card Suits
	public static final int SUIT_CLUBS = 0;
	public static final int SUIT_DIAMONDS = 1;
	public static final int SUIT_HEARTS = 2;
	public static final int SUIT_SPADES = 3;
	public static final int NUM_SUITS = 4;
	public static final int INVALID_CARD = -1;
	
	// Payouts and Strategies
	public static final int PAYOUT_ROYAL=0;
	public static final int PAYOUT_WILDRF=1;
	public static final int PAYOUT_NATSF=2;
	public static final int PAYOUT_STRFLUSH=3;
	public static final int PAYOUT_FIVE=4;
	public static final int PAYOUT_NATFOUR=5;
	public static final int PAYOUT_FOUR=6;
	public static final int PAYOUT_FULLHOUS=7;
	public static final int PAYOUT_FLUSH=8;
	public static final int PAYOUT_STRAIGHT=9;
	public static final int PAYOUT_THREE=10;
	public static final int PAYOUT_TWOPAIR=11;
	public static final int PAYOUT_PAIR=12;
	public static final int PAYOUT_NADA=13;
	public static final int STRATEGY_START=13;
	public static final int STRATEGY_ROYAL4=13;
	public static final int STRATEGY_ROYAL3=14;
	public static final int STRATEGY_ROYAL3AK=15;
	public static final int STRATEGY_ROYAL2=16;
	public static final int STRATEGY_ROYAL2AK=17;				// AK royal
	public static final int STRATEGY_ROYAL2T=18;				// with TEN
	public static final int STRATEGY_ROYAL2NOT=19;					// no TEN
	public static final int STRATEGY_ROYAL2DK=20;					// DEUCEs king special case
	public static final int STRATEGY_NATSF4=21;
	public static final int STRATEGY_NATSF4_1=22;
	public static final int STRATEGY_NATSF3x=23;
	public static final int STRATEGY_STRFLUSH4=24;
	public static final int STRATEGY_STRFLUSH4_1=25;
	public static final int STRATEGY_STRFLUSH4_2=26;
	public static final int STRATEGY_STRFLUSH4_2K1=27;
	public static final int STRATEGY_STRFLUSH4_3=28;
	public static final int STRATEGY_STRFLUSH3=29;
	public static final int STRATEGY_STRFLUSH3_1=30;
	public static final int STRATEGY_STRFLUSH3_2=31;
	public static final int STRATEGY_STRFLUSH3J1=32;		// 1 card J or better
	public static final int STRATEGY_STRFLUSH3_1J1=33;		// 1 card J or better
	public static final int STRATEGY_STRFLUSH3_1J2=34;		// 2 cards J or
	public static final int STRATEGY_STRFLUSH3_2J1=35;		// 1 card J or better
	public static final int STRATEGY_STRFLUSH3_2K1=36;		// 1 card K or better
	public static final int STRATEGY_STRFLUSH3_2J2=37;		// 2 cards J or better
	public static final int STRATEGY_STRFLUSH3_3=38;
	public static final int STRATEGY_STRFLUSH2=39;
	public static final int STRATEGY_STRFLUSH2_1=40;
	public static final int STRATEGY_STRFLUSH2_2=41;
	public static final int STRATEGY_STRFLUSH2_3=42;
	public static final int STRATEGY_FLUSH4=43;
	public static final int STRATEGY_FLUSH4_K1=44;
	public static final int STRATEGY_FLUSH3_AK=45;			// AK flush
	public static final int STRATEGY_FLUSH3_K1=46;
	public static final int STRATEGY_FLUSH3_J2=47;
	public static final int STRATEGY_FLUSH3_J1=48;
	public static final int STRATEGY_FLUSH3=49;
	public static final int STRATEGY_STRAIGHT4 =50;;
	public static final int STRATEGY_STRAIGHT4J1=51;		// 1 card J or better
	public static final int STRATEGY_STRAIGHT4_1=52;
	public static final int STRATEGY_STRAIGHT4_1AK =53;	// AK 4 card straight
	public static final int STRATEGY_STRAIGHT4_1J3=54;		// 3 cards J or better
	public static final int STRATEGY_STRAIGHT4_1J2=55;		// 2 cards J or better
	public static final int STRATEGY_STRAIGHT4_1J1=56;		// 1 cards J or better
	public static final int STRATEGY_STRAIGHT3=57;			// for KQJ
	public static final int STRATEGY_STRAIGHT2=58;		    // for QJ
	public static final int STRATEGY_THREE=59;				// needed if 3 of a kind taken over a payout
	public static final int STRATEGY_TWOPAIR=60;
	public static final int STRATEGY_PAIR=61;
	public static final int STRATEGY_HIGHCARD2=62; 		// use 2 high cards
	public static final int STRATEGY_HIGHCARD=63;
	public static final int PAYOUT_NONE=64;
	
	// misc
	public static final int MAX_CARDS = 54;
	public static final int NUM_CARDS = 5;
	public static final int JACKPOT_THRESHOLD = 50;	// per coin bet threshold for a jackpot
	public static final int MAX_BET = 5;
	
	//#ifdef PLACEHOLDER 
	/*
	 * Below is temporary code to keep Eclipse or other IDE's happy.  
	 */
	
	public static int BIT(int x) {
		return (int)(1 << x);
	}
	
	public static final int BITALL		=(0x3fff);			// does not include wild cards
	public static final int BIT_AKQJT9876 =(BIT(ACE) | BIT(KING) | BIT(QUEEN) | BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX));
	public static final int BIT_AKQJT987 =(BIT(ACE) | BIT(KING) | BIT(QUEEN) | BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN));
	public static final int	BIT_AKQJ	=(BIT(ACE) | BIT(KING) | BIT(QUEEN) | BIT(JACK));
	public static final int	BIT_AKQ		=(BIT(ACE) | BIT(KING) | BIT(QUEEN));
	public static final int	BIT_AK		=(BIT(ACE) | BIT(KING));
	public static final int	BIT_AJ		=(BIT(ACE) | BIT(JACK));
	public static final int	BIT_A		=(BIT(ACE));
	public static final int BIT_KQJT98765 =(BIT(KING) | BIT(QUEEN) | BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX) | BIT(FIVE));
	public static final int BIT_KQJT987	=(BIT(KING) | BIT(QUEEN) | BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN));
	public static final int	BIT_KQJ		=(BIT(KING) | BIT(QUEEN) | BIT(JACK));
	public static final int	BIT_KQ		=(BIT(KING) | BIT(QUEEN));
	public static final int	BIT_K_5ALOW	=(BIT(KING) | BIT(FIVE) | BIT(ACELOW));
	public static final int	BIT_K4		=(BIT(KING) | BIT(FOUR));
	public static final int	BIT_K		=(BIT(KING));
	public static final int BIT_QJT98765 =(BIT(QUEEN) | BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX) | BIT(FIVE));
	public static final int BIT_QJT9876	=(BIT(QUEEN) | BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX));
	public static final int BIT_QJT987	=(BIT(QUEEN) | BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN));
	public static final int	BIT_QJ		=(BIT(QUEEN) | BIT(JACK));
	public static final int	BIT_Q		=(BIT(QUEEN));
	public static final int BIT_JT987654 =(BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX) | BIT(FIVE) | BIT(FOUR));
	public static final int BIT_JT98765	=(BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX) | BIT(FIVE));
	public static final int BIT_JT9876	=(BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX));
	public static final int BIT_JT987	=(BIT(JACK) | BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN));
	public static final int	BIT_J		=(BIT(JACK));
	public static final int BIT_T98765432 =(BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX) | BIT(FIVE) | BIT(FOUR) | BIT(THREE) | BIT(TWO));
	public static final int BIT_T98765	=(BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX) | BIT(FIVE));
	public static final int BIT_T987	=(BIT(TEN) | BIT(NINE) | BIT(EIGHT) | BIT(SEVEN));
	public static final int BIT_NO_AK	=(BITALL & ~BIT(ACE) & ~BIT(KING) & ~BIT(ACELOW));
	public static final int BIT_NO_A	=(BITALL & ~BIT(ACE) & ~BIT(ACELOW));
	public static final int BIT_NO_ALOW	=(BITALL & ~BIT(ACELOW));
	public static final int BIT_T		=(BIT(TEN));
	public static final int BIT_98765	=(BIT(NINE) | BIT(EIGHT) | BIT(SEVEN) | BIT(SIX) | BIT(FIVE));
	public static final int BIT_9		=(BIT(NINE));
	public static final int BIT_876		=(BIT(EIGHT) | BIT(SEVEN) | BIT(SIX));
	public static final int BIT_65		=(BIT(SIX) | BIT(FIVE));
	public static final int BIT_6		=(BIT(SIX));
	public static final int BIT_54ALOW	=(BIT(FIVE) | BIT(FOUR) | BIT(ACELOW));
	public static final int	BIT_5ALOW	=(BIT(FIVE) | BIT(ACELOW));
	public static final int BIT_5		=(BIT(FIVE));
	public static final int BIT_432A	=(BIT(FOUR) | BIT(THREE) | BIT(TWO) | BIT(ACE));
	public static final int BIT_432		=(BIT(FOUR) | BIT(THREE) | BIT(TWO));
	public static final int BIT_4		=(BIT(FOUR));
	public static final int BIT_2 	=(BIT(TWO));	
	
	//#else
	//#include ${build.file.defs.etc}/bit.h
	//#endinclude
	/*
	 * ************ End of Special Precompile ******************
	 */
	//#endif
	
	public static final int HOLD_1		= 0x00000001;		// cards that should be held
	public static final int HOLD_2		= 0x00000002;
	public static final int HOLD_3		= 0x00000004;
	public static final int HOLD_4		= 0x00000008;
	public static final int HOLD_5		= 0x00000010;
	public static final int HOLD_ALL	= 0x0000001F;		// all cards should be held
	
	public static final int DRAWHOLD_1	= 0x00000100;		// cards that are held
	public static final int DRAWHOLD_2	= 0x00000200;
	public static final int DRAWHOLD_3	= 0x00000400;
	public static final int DRAWHOLD_4	= 0x00000800;
	public static final int DRAWHOLD_5	= 0x00001000;
	public static final int DRAWHOLD_ALL =0x00001f00;
	
	public static final int DRAW_SHIFT	= 8;				// shift to get DRAWHOLD_x to HOLD_x
	public static final int HOLDERROR_1 = 0x00010000;		// errors that should be shown
	public static final int HOLDERROR_2 = 0x00020000;
	public static final int HOLDERROR_3 = 0x00040000;
	public static final int HOLDERROR_4 = 0x00080000;
	public static final int HOLDERROR_5 = 0x00100000;
	
	public static final int HOLDERROR_SHIFT = 16;			// shift to get HOLDERROR_x to HOLD_x
	public static final int DRAWERROR_1 = 0x01000000;		// errors that should be shown
	public static final int DRAWERROR_2 = 0x02000000;
	public static final int DRAWERROR_3 = 0x04000000;
	public static final int DRAWERROR_4 = 0x08000000;
	public static final int DRAWERROR_5 = 0x10000000;
	
	public static final int DRAWERROR_SHIFT = 24;			// shift to get DRAWERROR_x to HOLD_x
	public static final int ERROR_1		= HOLDERROR_1 | DRAWERROR_1;
	public static final int ERROR_2		= HOLDERROR_2 | DRAWERROR_2;
	public static final int ERROR_3		= HOLDERROR_3 | DRAWERROR_3;
	public static final int ERROR_4		= HOLDERROR_4 | DRAWERROR_4;
	public static final int ERROR_5		= HOLDERROR_5 | DRAWERROR_5;
	public static final int ERROR_ALL	= 0xffff0000;		// mask for any errors
	
	public static final int[] DRAWHOLD = {DRAWHOLD_1,DRAWHOLD_2,DRAWHOLD_3,DRAWHOLD_4,DRAWHOLD_5};
	
	
	/*
	 * -------------------- Game Information ---------------------------------------------------------
	 * 
	 * Below are the registers for the gamelist (dynamic at compiletime) and current game information.
	 * 
	 */
	
    public static final int NUM_GAMES = 13;	// XXX should this be passed from the build?
    public static final String[] GAME_LIST = {
			"Jacks or Better",
			"Deuces Wild",
			"Joker Wild",
			"Double Bonus",
			"Dbl. Dbl. Bonus",
			"Dbl. Dbl. Jackpot",
			"Joker Wild 2 Pair",
			"Deuces Deluxe",
			"Bonus Dueces",
			"Loose Dueces",
			"Double Joker",
			"Vegas Dbl. Joker",
			"Kings or Better"
	};
    
	// Game is a tournament
    public static final int TOURNAMENT = NUM_GAMES;
	
	/*
	 * --------------- Persistant Game Settings -----------------------------------------------------------
	 * 
	 * Set by loadGameSettings();
	 * Saved By saveGame();
	 * 
	 */
	public static final int LOOP = -1;
	
	public static final int SETTING_AV = 0;
	public static final int SETTING_HELP = 1;
	public static final int SETTING_ACTIVE = 2;
	public static final int SETTING_SCORES = 3;

	// Sound/Vibrate State
	public static final int AV_OFF = 0;
	public static final int AV_VIBRATE = 1;
	public static final int AV_SOUND = 2;
	public static final int AV_ALL = 3;
	public static int avSetting = AV_OFF;
	
	// Show Game Help
	public static final int HELPER_OFF = 0;
	public static final int HELPER_ON = 1;
	public static int hSetting = HELPER_ON;
	
	// Current Game Scores
	public static boolean[] gamePlayed = new boolean[NUM_GAMES];
	
	// INFORMATIONAL CONSTANT, DO NOT MODIFY OUTSIDE OF THIS CLASS
	
	/*
	 * -------------------- Local Game Information ---------------------------------------------------
	 * 
	 * Below are the Tables and other information for the current game.
	 * 
	 */
	
	public static final int STATE_PREGAME = -1;
	public static final int STATE_BET = 0;
	public static final int STATE_DEAL = 1;
	public static final int STATE_PICK_CARDS = 2;
	public static final int STATE_CORRECT = 3;
	public static final int STATE_DRAW = 4;
	public static final int STATE_BANNER = 5;
	public static final int STATE_FINISH = 6;
	public static final int STATE_ENDTOURNY = 7;
	public static int gState;
	
	// Current Game Definitions
	public static final int DEF_NUMJOKERS = 0;
	public static final int DEF_WILDSPOTS = 1;
	public static final int DEF_TOPFONTCOLOR = 2;
	public static final int DEF_BACKGROUND = 3;
	public static int[] tGameDef;

	// Current Game PayBack
	public static final int PAY_WHOLE=0;
	public static final int PAY_FRAC=1;
	public static int[] tPayBack;
	
	// Current Game Change Reference
	public static final int CH_AMOUNT = 0;
	public static final int CH_WHOLE = 1;
	public static final int CH_FRAC = 2;
	public static int[][] tChanges;
	
	// Current Game Payoffs
	public static Payoff[] tPayoff;
	
	// Current Game Strategies
	public static Strategy[] tStrategy;
	
	// Current Game Strings
	public static final int STR_NAME = 0;
	public static final int STR_PAYOFFS = 1;
	public static final int STR_STRATEGIES = 2;
	public static String[][] gStrings;
	
	//Public Card Slots (used for Display purposes)
	public static Deck deck = new Deck();	
	public static Hand hand = new Hand();
	
	public static int 		nBet;
	
	public static int		nCoins;					// number of coins (starts at START_COINS)
	public static int		nCoinsWon;				// number of coins won or lost (starts at 0)
	public static int		nCoinsBet;				// amount wagered
	public static int		nHands;					// hands played
	public static int		nHandsWon;				// hands we won 
	public static int		nErrors;				// errors made

	public static int		nSeconds;				// time played (seconds)
	public static int		nLastWin;				// current coins won (0=none)			
	public static int		nSelect;				// what button is selected (see SELECT_xxx)
	
	public static int		nPayoff;				// index public static into payoff table
	public static int 		tPayoffSize;
	public static int 		nPayoffType;
	
	public static int 		nStrategy;					// strategy we should use (see enum above)
	public static int 		tStrategySize;				// number of lines used in tStrategy[]
	public static int 		nStrategyStart;				// which line are we starting on for display purposes
	public static int 		nStrategyLine;				// which strategy we are using use (index into tStrategy[])
		
	public static int 		nBannerWins;
	
	public static int 		nSpot;
	public static int 		nHold;
    public static int[] 	nWins;

    public static int wildCard = NONE;
	public static int numJokers,numWilds;
	public static int activeGame = NONE;
	
	// Random Number Generator
	//private RandInt random = new RandInt();
	public static RandInt random = new RandInt();
	
	public static GameScreen pokerScreen;
	public static int curGameType = NONE;
	
	
	/*
	 * ----------------- Primary Game-Driver Methods -----------------------------------------
	 * 
	 * These methods are only concerned with the operation of whichever
	 * particular game is currently active.
	 * 
	 */
	
	public static void setGame(int selectedGame) {
		curGameType = selectedGame;
		if (selectedGame == TOURNAMENT) {
			activeGame = TournInfo.nGame;
			//#ifdef DEBUG
			System.out.println("Set Game: Tournament: " + GAME_LIST[activeGame] + " ");
			//#endif
		} else {
			activeGame = selectedGame;
			//#ifdef DEBUG
			System.out.println("Set Game: Regular: " + GAME_LIST[activeGame] + " ");
			//#endif
		}
		try {
			InputStream iStream = new Object().getClass().getResourceAsStream("/Game_"+activeGame+".vpg");
			DataInputStream reader = new DataInputStream(iStream);
			
			int dataLength = reader.readInt();
			byte[] inData = new byte[dataLength];
			gStrings = new String[3][];
			gStrings[0] = new String[1];
			gStrings[0][0] = new String();
			for (int c = 0; c < dataLength; c++) gStrings[0][0]+=((char)reader.readByte());
			
			tGameDef = new int[reader.readInt()]; 
			for (int c = 0; c < tGameDef.length; c++) tGameDef[c] = reader.readShort();
			
			tPayBack = new int[reader.readInt()];
			for (int c = 0; c < tPayBack.length; c++) tPayBack[c] = reader.readShort();
			
			tChanges = new int[reader.readInt()][3];
			for (int c = 0; c < tChanges.length; c++) {
				for (int d = 0; d < 3; d++) tChanges[c][d] = reader.readShort();
			}
			
			tPayoffSize = reader.readInt();
			tPayoff = new Payoff[tPayoffSize];
			inData = new byte[Payoff.SIZE];
			for (int c = 0; c < tPayoffSize; c++) {
				reader.read(inData,0,Payoff.SIZE);
				tPayoff[c] = new Payoff(inData);
			}
			    
			tStrategySize = reader.readInt();
			tStrategy = new Strategy[tStrategySize];
			inData = new byte[Strategy.SIZE];
			for (int c = 0; c < tStrategySize; c++) {
				reader.read(inData,0,Strategy.SIZE);
				tStrategy[c] = new Strategy(inData);
			}
			    
		    for (int c = 1; c < 3; c++) {
		    	gStrings[c] = new String[reader.readInt()];
		        for (int d = 0; d < gStrings[c].length;d++) {
		        	gStrings[c][d] = Strings.STRAT[reader.readShort()];
		        }
		    }
		    
		    reader.close();
		} catch (Exception e) {
			//#ifdef DEBUG
			System.out.println("Load Exception: " + e);
			//#endif
		}
		
		
	    wildCard = tGameDef[DEF_WILDSPOTS];
		numJokers = tGameDef[DEF_NUMJOKERS];
		if (tGameDef[DEF_TOPFONTCOLOR] == GPCanvas.BLACK) {
			GameScreen.topStringFont = GameScreen.SMALL_BLACK;
		} else {
			GameScreen.topStringFont = GameScreen.SMALL_WHITE;
		}
		
		if (wildCard != NONE) {
			numWilds = numJokers + 4;
		} else {
			numWilds = numJokers;
		}
		
		nWins = new int[tPayoffSize];
		deckInitialize(wildCard,numJokers,numWilds);
		hand = new Hand();
		
	    // Try and load previous data, if fail, make a new record
	    boolean needNew = false;
	    
		try {
			RecordStore load;

			if (selectedGame == TOURNAMENT) {
				load = RecordStore.openRecordStore(Strings.RECORD_TOURNAMENT,true);
			} else {
				load = RecordStore.openRecordStore(Strings.RECORD_HEADER + gStrings[0][0],true);
			}
			    
		    // Try and load Data
			try {
			    
			    ByteArrayInputStream iStream = new ByteArrayInputStream(load.getRecord(1));
				DataInputStream reader = new DataInputStream(iStream);
				
				int nGame = reader.readInt();					// Tournament number (-1 if not)
				
				if (selectedGame == TOURNAMENT && nGame != TournInfo.nGame) needNew = true;
				
				nCoins = reader.readInt();					// number of coins (starts at START_COINS)
				nCoinsWon = reader.readInt();				// number of coins won or lost (starts at 0)
				nCoinsBet = reader.readInt();				// amount wagered
				nSeconds = reader.readInt();				// time played (seconds)
				nHold = reader.readInt();
				
				nHands = reader.readShort();					// hands played
				nHandsWon = reader.readShort();				// hands we won 
				nErrors = reader.readShort();				// errors made
				
				gState = reader.readByte();
				nLastWin = reader.readByte();				// current coins won (0=none)	
				nSelect = reader.readByte();				// what button is selected (see SELECT_xxx)
				nPayoff = reader.readByte();				// index o payoff table
				nStrategyLine = reader.readByte();			// which strategy we are using use (index o pStrategyTable[])
				nSpot = reader.readByte();
				nBet = reader.readByte();					// amount we are betting
				
				int c;
				
				for (c = 0; c < hand.info.length; c++) {
					hand.info[c].suit = reader.readByte();
					hand.info[c].spot = reader.readByte();
				}
				for (c = 0; c < deck.cards.length; c++) {
				    deck.cards[c].suit = reader.readByte();
					deck.cards[c].spot = reader.readByte();
				}
				for (c = 0; c < nWins.length; c++) {
					nWins[c] = reader.readShort();	// wins for each type
				}
				if (gState >= STATE_DEAL) {
				    // If we're mid-game, evaluate the hand
				    orderHand();
				    evaluateHand(hand);
				}
			} catch (Exception e) {
			    needNew = true;
			}

			load.closeRecordStore();
			
		} catch (Exception e) {
			//#ifdef DEBUG
			System.out.println("GameInit: Load: " + e);
			//#endif
			needNew = true;
		}
		
		if (needNew) {	
			createNew();
		} else {
			//#ifdef DEBUG
			System.out.println("SUCCESSFUL");
			//#endif
		}
		
		if (gState > Game.STATE_BET) {
		    orderHand();
		    evaluateHand(hand);
		}
		
	}

	public static void PickHint(int strat) {
		
		switch(tStrategy[strat].type) {
		case STRATEGY_ROYAL4:
		case STRATEGY_ROYAL3:
		case STRATEGY_ROYAL3AK:
		case STRATEGY_ROYAL2:
		case STRATEGY_ROYAL2AK:		
		case STRATEGY_ROYAL2T: 
		case STRATEGY_ROYAL2NOT:	
		case STRATEGY_ROYAL2DK:	
			pokerScreen.infoStrings[0] = Strings.STR_ERR9_1;
			pokerScreen.infoStrings[1] = Strings.STR_ERR9_2;
			pokerScreen.infoStrings[2] = Strings.STR_ERR9_3;
			break;
		
		case STRATEGY_NATSF4:
		case STRATEGY_NATSF4_1:
		case STRATEGY_NATSF3x:
			pokerScreen.infoStrings[0] = Strings.STR_THISNATURAL;			// This natural str flush
			pokerScreen.infoStrings[1] = Strings.STR_STRFLUSH;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;

		case STRATEGY_STRFLUSH4:
			pokerScreen.infoStrings[0] = Strings.STR_4C;	
			pokerScreen.infoStrings[1] = Strings.STR_STRFLUSH;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;
		case STRATEGY_STRFLUSH4_1:
		case STRATEGY_STRFLUSH4_2:
		case STRATEGY_STRFLUSH4_2K1:
		case STRATEGY_STRFLUSH4_3:
			pokerScreen.infoStrings[0] = Strings.STR_4CINSIDE;	
			pokerScreen.infoStrings[1] = Strings.STR_STRFLUSH;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;

		case STRATEGY_STRFLUSH3:
		case STRATEGY_STRFLUSH3J1:				
			pokerScreen.infoStrings[0] = Strings.STR_3C;	
			pokerScreen.infoStrings[1] = Strings.STR_STRFLUSH;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;
		case STRATEGY_STRFLUSH3_1:
		case STRATEGY_STRFLUSH3_2:
		case STRATEGY_STRFLUSH3_1J1:			
		case STRATEGY_STRFLUSH3_1J2:			
		case STRATEGY_STRFLUSH3_2J1:			
		case STRATEGY_STRFLUSH3_2K1:		
		case STRATEGY_STRFLUSH3_2J2:		
		case STRATEGY_STRFLUSH3_3:
			pokerScreen.infoStrings[0] = Strings.STR_3CINSIDE;	
			pokerScreen.infoStrings[1] = Strings.STR_STRFLUSH;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;

		case STRATEGY_STRFLUSH2:
			pokerScreen.infoStrings[0] = Strings.STR_2C;	
			pokerScreen.infoStrings[1] = Strings.STR_STRFLUSH;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;
		case STRATEGY_STRFLUSH2_1:
		case STRATEGY_STRFLUSH2_2:
		case STRATEGY_STRFLUSH2_3:
			pokerScreen.infoStrings[0] = Strings.STR_2CINSIDE;	
			pokerScreen.infoStrings[1] = Strings.STR_STRFLUSH;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;

		case STRATEGY_STRAIGHT4:
		case STRATEGY_STRAIGHT4J1:			
			pokerScreen.infoStrings[0] = Strings.STR_4C;	
			pokerScreen.infoStrings[1] = Strings.STR_STRAIGHT;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;
		case STRATEGY_STRAIGHT4_1:
		case STRATEGY_STRAIGHT4_1AK:		
		case STRATEGY_STRAIGHT4_1J3:		
		case STRATEGY_STRAIGHT4_1J2:	
		case STRATEGY_STRAIGHT4_1J1:			
			pokerScreen.infoStrings[0] = Strings.STR_4CINSIDE;	 
			pokerScreen.infoStrings[1] = Strings.STR_STRAIGHT;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;
		case STRATEGY_STRAIGHT3:				
			pokerScreen.infoStrings[0] = Strings.STR_3C;	
			pokerScreen.infoStrings[1] = Strings.STR_STRAIGHT;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;
		case STRATEGY_STRAIGHT2:			
			pokerScreen.infoStrings[0] = Strings.STR_2C;	
			pokerScreen.infoStrings[1] = Strings.STR_STRAIGHT;
			pokerScreen.infoStrings[2] = Strings.STR_ERR_3[(nHands % 3)];
			break;

		case STRATEGY_FLUSH4:
		case STRATEGY_FLUSH4_K1:
		case STRATEGY_FLUSH3_AK:				
		case STRATEGY_FLUSH3_K1:
		case STRATEGY_FLUSH3_J2:
		case STRATEGY_FLUSH3_J1:
		case STRATEGY_FLUSH3:
			pokerScreen.infoStrings[0] = Strings.STR_THISFLUSH;	// this flush is the best choice
			pokerScreen.infoStrings[1] = Strings.STR_ERR_3[3];
			break;

		case STRATEGY_PAIR:
			pokerScreen.infoStrings[0] = Strings.STR_ERR8_1;		// hold a pair
			pokerScreen.infoStrings[1] = Strings.STR_ERR8_2;
			break;

		default:
			pokerScreen.infoStrings[0] = Strings.STR_ERR7_1;		// hold these for best chance
			pokerScreen.infoStrings[1] = Strings.STR_ERR7_2;
			break;
		}
	}
	
	/*
	 * ------------------ Poker Game Methods -------------------------------------------------
	 * 
	 * These Methods Maintain the poker fundamentals- Dealing, Evaluating and Offering Strategy
	 * 
	 */
		

	//
	//	Stuff the deck with ordered cards
	//
	public static void deckInitialize(int wildspot, int jokers, int wildcards) {
	    
		
		//System.out.println("GAME: Wild:" + wildspot + " JOKERS:" + jokers + " #WC:" + wildcards);
		
		deck = new Deck();
		
		int CARDS = 52;
	
		deck.wildspot = wildspot;
		deck.jokers = jokers; 
		deck.wildcards = wildcards;
		deck.numCards = CARDS + jokers;
		
		int i;
		
		for (i = 0; i < CARDS; i++) {
			deck.cards[i] = new Card((i/13),(i%13)+TWO);
		}
		for (i = 0; i < jokers; i++) {
			deck.cards[i+CARDS] = new Card(0,JOKER);				// must be 0 for swapping cards to work right
		}
		
		
	}

	/*
	 * Replaces Unheld Cards
	 */
	public static void deal() {
		
		for (int i = 0; i < 5; i++) {
			if ((nHold>>>(DRAW_SHIFT+i) & 1) == 0) {
				Card	temp;
				// swap cards with new card
				temp = new Card(deck.cards[i].suit,deck.cards[i].spot);
				deck.cards[i] = new Card(deck.cards[i + NUM_CARDS].suit,deck.cards[i+NUM_CARDS].spot);		
				deck.cards[i + NUM_CARDS] = new Card(temp.suit,temp.spot);			
			}
		}
	}

	//
	//		Quickly shuffle the "count" number of cards.
	//
	//		NOTE:	If you swap card i with any card from 0 to numcards we will get an uneven distribution
	//				You must swap i with the cards remaining.
	//
	public static void deckShuffle(int count) {

		Card	temp;
		int		i;
		int 	num;
		
		for (i = 0; i < count; i++) {

			num = random.nextInt(deck.numCards - i)+i;
	
			temp = new Card(deck.cards[num].suit,deck.cards[num].spot);						// pick a card from the deck
			
			deck.cards[num] = new Card(deck.cards[i].suit,deck.cards[i].spot);					// exchange with this card in the deck
			deck.cards[i] = new Card(temp.suit,temp.spot);		
			
		}
		
	}
	
	//
	//	 order our hand from highest to lowest spot card excluding wild cards
	//
	public static void orderHand(){
	
		int		i, j, wild;
		
		hand = new Hand();
		
		wild = deck.wildspot;
		hand.nLast = -1;
		hand.nWilds = 0;
		hand.nWildHold = 0;
		hand.nWildSuits = 0;
		hand.nPair = -1;
		
		for (i = 0; i < NUM_CARDS; i++) {
			
			if (deck.cards[i].spot == wild){
			    
				hand.nWildSuits |= (1 << deck.cards[i].suit);
				hand.nWilds += 1;
				hand.nWildHold |= HOLD_1 << i;
				continue;
			}
			
			if (deck.cards[i].spot == JOKER) {
			   
				hand.nWilds += 1;
				hand.nWildHold |= HOLD_1 << i;		// hold all wild cards
				continue;
			}
			
			// sort highest to lowest 
			for (j = hand.nLast; j >= 0; j--) {
				
				if (hand.info[j].spot >= deck.cards[i].spot) {
					
					if (hand.info[j].spot == deck.cards[i].spot) {
						
						if (hand.nPair < 0 || hand.info[hand.nPair].spot < hand.info[j].spot) {
							
							hand.nPair = j;
						}
					}
					break;
				}
				
				
				hand.info[j+1].suit = hand.info[j].suit;
				hand.info[j+1].spot = hand.info[j].spot;
				hand.info[j+1].card = hand.info[j].card;
				if (hand.nPair == j) {		// we moved our highest pair
					hand.nPair = (j + 1);
				}
			}
			
			hand.info[j+1].suit = deck.cards[i].suit;
			hand.info[j+1].spot = deck.cards[i].spot;
			hand.info[j+1].card = i;
			hand.nLast += 1;
		}

	}


	public static void setPayoff(int type, int spot, int kickers, int holds) {
		int	i, end;
		
		int kickerFlags = kickers;
			
		end = nPayoff;						// stop at current payout
		if (end < 0) {
	 		end = tPayoffSize;				// or end of list
		}
		
		for (i = 0; i < end; i++) {
			if (tPayoff[i].type == PAYOUT_STRFLUSH
			&&  type == PAYOUT_NATSF				// if no natural straight flush in the payout
			&& (tPayoff[i].spotFlags & (1 << spot)) != 0
			&& (tPayoff[i].kickerFlags & kickerFlags) != 0) {
				type = PAYOUT_STRFLUSH;				// use the straight flush payout
			}
			if (tPayoff[i].type == PAYOUT_FOUR 
			&&  type == PAYOUT_NATFOUR				// if no natural four of a kind in the payout
			&& (tPayoff[i].spotFlags & (1 << spot)) != 0
			&& (tPayoff[i].kickerFlags & kickerFlags) != 0) {
				type = PAYOUT_FOUR;					// use the four of a kind payout
			}
			if (tPayoff[i].type == type
			&& (tPayoff[i].spotFlags & (1 << spot)) != 0
			&& (tPayoff[i].kickerFlags & kickerFlags) != 0) {
				nPayoff = i;
				nPayoffType = tPayoff[i].type; 
				nSpot = spot;					// save card info for later
				nHold &= ~HOLD_ALL;			// clear old and add new hold cards  
				if (tPayoff[i].kickerFlags != BITALL) {
					nHold |= HOLD_ALL;		// hold kicker to PAYOUT_FOUR
				} else {
					nHold |= holds;
				}
				return;
			}
		}
	}


	public static void evaluateHand(Hand hand)
	{
		int i, j, last, pairs, wilds, pairHold;

		// clear any previous stats and strategies
		nPayoff = -1;							// nothing to report so far
		nPayoffType = PAYOUT_NONE;
		last = hand.nLast;
		wilds = hand.nWilds;
		nHold &= ~HOLD_ALL;

		// set payoffs if any
		if (hand.nPair < 0) {						// no pairs
			j = hand.info[last].spot + last + wilds;
			if (j > ACE) {							// straights stop at ACE high
				j = ACE;
			}
			if (j >= hand.info[0].spot) {
				setPayoff(PAYOUT_STRAIGHT, j, BITALL, HOLD_ALL);
			} else if (hand.info[0].spot == ACE && last > 0 && hand.info[1].spot <= FIVE) {
				setPayoff(PAYOUT_STRAIGHT, FIVE, BITALL, HOLD_ALL);
			}
		}

		// check for other payouts now
		for (i = 0; i <= last; i++)		
		{
			hand.info[i].nSpots = 1;				// clear out the count 
			hand.info[i].nSuits = 1;
			hand.info[i].nSpotHold = 1 << hand.info[i].card;
			hand.info[i].nSuitHold = 1 << hand.info[i].card;
			if (hand.info[i].spot >= TEN)
			{
				hand.info[i].nRoyal = 1;
				hand.info[i].nRoyalHold = 1 << hand.info[i].card;
				hand.info[i].nRoyalSpots = BIT(hand.info[i].spot);
			}
			else
			{
				hand.info[i].nRoyal = 0;
				hand.info[i].nRoyalHold = 0;
				hand.info[i].nRoyalSpots = 0;
			}
		}
		
		pairs = NOSPOTS;
		pairHold = 0;
		for (i = 0; i <= last; i++)		
		{
			// walk through the list and count the number of spots, suits, royals, etc.
			for (j = i+1; j <= last; j++)
			{
				if (hand.info[i].spot == hand.info[j].spot)
				{
					hand.info[i].nSpots += 1;
					hand.info[j].nSpots += 1;
					hand.info[i].nSpotHold |= 1 << hand.info[j].card;
					hand.info[j].nSpotHold |= 1 << hand.info[i].card;
				}
				if (hand.info[i].suit == hand.info[j].suit)
				{
					hand.info[i].nSuits += 1;
					hand.info[j].nSuits += 1;
					hand.info[i].nSuitHold |= 1 << hand.info[j].card;
					hand.info[j].nSuitHold |= 1 << hand.info[i].card;
					if (hand.info[j].spot >= TEN)		// count others that are part of the royal
					{
						hand.info[i].nRoyal += 1;		// use just first card in royal
//							hand.info[j].nRoyal += 1;		// else KJ suited will pick the J as the highest card
						hand.info[i].nRoyalHold |= 1 << hand.info[j].card;
//							hand.info[j].nRoyalHold |= 1 << hand.info[i].card;
						hand.info[i].nRoyalSpots |= BIT(hand.info[j].spot);
					}
				}
			}
//				//MYPRINTF(("hand[%d]= %d %d count=%d %d card=%d hold=0x%x", 
//					i, hand.info[i].suit, hand.info[i].spot, hand.info[i].nSuits, hand.info[i].nSpots, hand.info[i].card, hand.info[i].nHold));
			if (hand.info[i].nSpots == 2)
			{
				if (hand.info[i].spot > pairs)			// first natural pair
				{
					pairs = hand.info[i].spot;
					pairHold = hand.info[i].nSpotHold;	// save hold for this pair
					if (wilds < 2						// in case four of kind does not rank better than some full house
					&& nPayoff >= 0				// we have a win already
					&& nPayoffType == PAYOUT_THREE)
					{
						setPayoff(PAYOUT_FULLHOUS, hand.info[i].spot, BITALL, HOLD_ALL);
					}
				}
				else if (hand.info[i].spot < pairs)	// second natural pair
				{
					if (wilds != 0)
						setPayoff(PAYOUT_FULLHOUS, hand.info[i].spot, BITALL, HOLD_ALL);
					else
					{
						setPayoff(PAYOUT_TWOPAIR, hand.info[i].spot, BITALL, pairHold | hand.info[i].nSpotHold);
					}
				}
			}
			if (hand.info[i].nSuits + wilds == 5)
			{
				if (nPayoffType == PAYOUT_STRAIGHT)
				{
					if (hand.info[last].spot >= TEN)
					{
						if (wilds > 0)
							setPayoff(PAYOUT_WILDRF, hand.info[i].spot, BITALL, HOLD_ALL);
						else
							setPayoff(PAYOUT_ROYAL, hand.info[i].spot, BITALL, HOLD_ALL);
					}
					else if (wilds == 0)
						setPayoff(PAYOUT_NATSF, hand.info[i].spot, BITALL, HOLD_ALL);
					else
						setPayoff(PAYOUT_STRFLUSH, hand.info[i].spot, BITALL, HOLD_ALL);
					// this assumes there is only one wild card of the same suit
					// the code below does not work for aces wild
					if (wilds == 1
					&& (hand.nWildSuits & (1 << hand.info[0].suit)) != 0
					&& ( ( deck.wildspot == hand.info[0].spot + 1 
						&& hand.info[0].spot == hand.info[hand.nLast].spot + 3)
					  || ( deck.wildspot == hand.info[hand.nLast].spot - 1 
						&& hand.info[0].spot == hand.info[hand.nLast].spot + 3)
					  || ( deck.wildspot > hand.info[hand.nLast].spot
					    && deck.wildspot < hand.info[0].spot
						&& hand.info[0].spot == hand.info[hand.nLast].spot + 4)
					  || ( hand.info[0].spot == ACE
					    && deck.wildspot <= FIVE
						&& hand.info[1].spot <= FIVE))) 
					{
						setPayoff(PAYOUT_NATSF, hand.info[i].spot, BITALL, HOLD_ALL);
					}
				}
				else
					setPayoff(PAYOUT_FLUSH, hand.info[i].spot, BITALL, HOLD_ALL);
			}
			if (wilds == 5)
				setPayoff(PAYOUT_FIVE, WILD, BITALL, hand.nWildHold);
			else if (wilds == 4)
				setPayoff(PAYOUT_FOUR, WILD, BITALL, hand.nWildHold);
			else if (wilds == 3)
				setPayoff(PAYOUT_THREE, WILD, BITALL, hand.nWildHold);
			if (hand.info[i].nSpots + wilds == 5)
				setPayoff(PAYOUT_FIVE, hand.info[i].spot, BITALL, HOLD_ALL);
			else if (hand.info[i].nSpots + wilds == 4) {
				for (j = hand.nLast; j >= 0; j--)	// use lowest card as a kicker
				{
					if (hand.info[j].nSpots == 1)	// we found the kicker
						break;
				}
				if (wilds == 0)
					setPayoff(PAYOUT_NATFOUR, hand.info[i].spot, BIT(hand.info[j].spot), hand.nWildHold | hand.info[i].nSpotHold);
				else
					setPayoff(PAYOUT_FOUR, hand.info[i].spot, BIT(hand.info[j].spot), hand.nWildHold | hand.info[i].nSpotHold);
			}
			else if (hand.info[i].nSpots + wilds == 3)
			{
				if (wilds == 0	
				&&  pairs != NOSPOTS)			// we already have a pair without a payoff
					setPayoff(PAYOUT_FULLHOUS, hand.info[i].spot, BITALL, HOLD_ALL);
				else
					setPayoff(PAYOUT_THREE, hand.info[i].spot, BITALL, hand.nWildHold | hand.info[i].nSpotHold);
			}
			else if (hand.info[i].nSpots + wilds == 2)
				setPayoff(PAYOUT_PAIR, hand.info[i].spot, BITALL, hand.nWildHold | hand.info[i].nSpotHold);
		}
	}


	public static int CountBits(int bits) {
		int	value = 0;
		for ( ; bits != 0; bits >>>= 1)
		{
			if ((bits & 1) != 0)
				value += 1;
		}
		return value;
	}


	//		Returns true if cards held are OK
	//		Returns false if cards held are still wrong
	//
	public static boolean RecheckStrategy(Hand hand) {
		Hand held = new Hand();							// new hand of hand cards only
		int	nOldHold = nHold;
		int	hold;							// which cards were held
		int	i, j;

		hold = (nHold >>> DRAW_SHIFT) & HOLD_ALL;	// cards we should hold

		if (nPayoffType == PAYOUT_NATFOUR	// we have four of a kind with no wild cards possible
		&&  deck.wildcards == 0			// accept holding all 5 cards
		&& (nHold & DRAWHOLD_ALL) == DRAWHOLD_ALL)
		{
			nHold = DRAWHOLD_ALL | HOLD_ALL;	// clear any errors
			return true;
		}

//			This code is not sufficient because we could egt a payout of 3 K's
//			when 3 9"s is best.  For example, with Joker, Joker, K, 9, 8.
	//
//			if (nPayoffType < STRATEGY_START			// payoff is best strategy
//			|| (nWildHold & (int)hold) != nWildHold)	// didn't hold the wild cards so forget it
//				return false;								// no need to check for anything more, the player made an error

		// copy cards hand into new hand
		held.nWilds = hand.nWilds;
		held.nWildHold = hand.nWildHold;
		held.nLast = -1;
		held.nPair = -1;
		for (i = 0; i < NUM_CARDS; i++)
		{
			if ((hold & (1 << hand.info[i].card)) != 0)
			{
				j = held.nLast += 1;
				held.info[j].spot = hand.info[i].spot;
				held.info[j].suit = hand.info[i].suit;
				held.info[j].card = hand.info[i].card;
				held.info[j].nSpotHold = hand.info[i].nSpotHold & hold;
				held.info[j].nSuitHold = hand.info[i].nSuitHold & hold;
				held.info[j].nRoyalHold = hand.info[i].nRoyalHold & hold;
	//MYPRINTF((sDebug, "spothold[%d]=0x%x (was spothold[%d]=0x%x", j, held.info[j].nSpotHold, i, info[i].nSpotHold));
				held.info[j].nSpots = CountBits(held.info[j].nSpotHold);
				held.info[j].nSuits = CountBits(held.info[j].nSuitHold);
				held.info[j].nRoyal = CountBits(held.info[j].nRoyalHold);
				held.info[j].nRoyalSpots = 0;		// no need to worry about Royals
	//MYPRINTF((sDebug, "spots[%d]=0x%x", j, held.info[j].nSpots));
				if (held.info[j].nSpots == 2)
					held.nPair = j;
			}
		}
	//MYPRINTF((sDebug, "RECHECKING hold=0x%x nPair=%d last index=%d", hold, held.nPair, held.nLast));

		if (getStrategy(held, nStrategyLine, true))
		{
			// check for any extra cards
			int errors;
	//MYPRINTF((sDebug, "RECHECK should hold=0x%x held=0x%x", nHold & HOLD_ALL, hold));
			errors = (nHold & HOLD_ALL) & ~hold;
			if (errors != 0)								// we have draw errors (user said draw when it should hold)
			{
				nHold |= errors << DRAWERROR_SHIFT;
			}
			errors = ~(nHold & HOLD_ALL) & hold;
			if (errors != 0)								// we have hold errors (user said hold when it should draw)
			{
				nHold |= errors << HOLDERROR_SHIFT;
			}
			nHold |= hold << DRAW_SHIFT;		// add back held cards 
			if ((nHold & ERROR_ALL) != 0)			// if there any errors
			{
				return false;
			} 
			else									// strategy is still valid
			{
				return true;
			}
		}
	//MYPRINTF((sDebug, "RECHECKING FAILED!"));
		nHold |= nOldHold;					// add back held cards and errors in case nHold changed
		return false;								// strategy is still wrong so tell the player
	}


	public static boolean getStrategy(Hand hand, int index, boolean oneshot) {
		int		last = hand.nLast;
		int		wilds = hand.nWilds;
		int		wildhold = hand.nWildHold;
		int		type = 0;
		int		num = 0;
		int holes = 0;
		int n = 0;
		int low = 0;
		int next =0;
		int hold = 0; 
		int i = 0;
		int j = 0;
		int		needed = 0;
		int needed2 = 0; 
		int highcard = 0;

		for (index = 0; index < tStrategy.length;index++) {
			if (tStrategy[index].wilds <= wilds) {
				break;
			}
		}
		
		//System.out.println("Starting at "+ index);
		
		while (index < tStrategySize)
		{
			type = tStrategy[index].type;
			if (type == nPayoffType)
			{
				if (hand.nPair >= 0)
					n = hand.info[hand.nPair].spot;
				else 
					n = -1;					// in case we have two wild cards
				if (type == PAYOUT_FOUR)	// make sure the right four of a kind is used
				{
					if (hand.nPair == 0)
						next = hand.info[last].spot;
					else
						next = hand.info[0].spot;
					if ((tStrategy[index].spotFlags & BIT(n)) == 0
					||  (tStrategy[index].kickerFlags & BIT(next)) == 0)
						if (CONTINUE(index,hand.nWilds,hand.nWildHold)) return true;
				}
				else if ((type == PAYOUT_THREE && hand.nWilds == 2)
				||		 (type == PAYOUT_PAIR  && hand.nWilds == 1))
				{							// make sure the right pair or three of a kind is used
					if (n >= 0)
					{
						if ((tStrategy[index].spotFlags & BIT(n)) == 0)
							if (CONTINUE(index,hand.nWilds,hand.nWildHold)) return true;
					}
					else  					// we have two wild cards with three of a kind 
					{						// or 1 wild card with a pot card
						for (i = 0; i <= hand.nLast; i++)
						{
							n = hand.info[i].spot;
							if ((tStrategy[index].spotFlags & BIT(n)) != 0)
							{
								nHold = wildhold | (1 << hand.info[i].card);
								nStrategyLine = index;
								nStrategy = type;		// we found the payout entry so return
								return true;
							}
						}
						if (CONTINUE(index,hand.nWilds,hand.nWildHold)) return true;
					}
				}
				nStrategyLine = index;
				nStrategy = type;		// we found the payout entry so return
				return true;
			}
			if (type >= STRATEGY_START)		// this is a strategy so test (skip payoffs)
			{
				needed = 0;					// count of face cards needed
	//MYPRINTF((sDebug, "Check index=%d strategy=%d", index, type));
				switch(type)
				{
				case STRATEGY_ROYAL4:
					num = 4 - wilds;
					if (ROYAL(i, j, last, num, hand.info, index, type, wildhold)) return true;
					break;
				case STRATEGY_ROYAL3AK:		// AKx royal
				case STRATEGY_ROYAL3:
					num = 3 - wilds;
					if (ROYAL(i, j, last, num, hand.info, index, type, wildhold)) return true;
					break;
				case STRATEGY_ROYAL2AK:		// AK royal
				case STRATEGY_ROYAL2T:		// with TEN
				case STRATEGY_ROYAL2NOT:	// no TEN
				case STRATEGY_ROYAL2:
				case STRATEGY_ROYAL2DK:
					num = 2 - wilds;
					if (ROYAL(i, j, last, num, hand.info, index, type, wildhold)) return true;
					break;

				case STRATEGY_NATSF4:
					holes = 0;
					num = 3;
					if (NATSF(i, last, num, needed2, needed, hand.info, index, hand.nWildSuits,holes, low, n, highcard, type, wildhold,hold, j)) return true;
					break;
				case STRATEGY_NATSF4_1:
					holes = 1;
					num = 3;
					if (NATSF(i, last, num, needed2, needed, hand.info, index, hand.nWildSuits,holes, low, n, highcard, type, wildhold,hold, j)) return true;
					break;
				case STRATEGY_NATSF3x:						// wild + 
					holes = 0;
					num = 2;
					if (NATSF(i, last, num, needed2, needed, hand.info, index, hand.nWildSuits,holes, low, n, highcard, type, wildhold,hold, j)) return true;
					break;

				case STRATEGY_STRFLUSH4:
					num = 4 - wilds;
					holes = 0;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH4_1:
					num = 4 - wilds;
					holes = 1;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH4_2:
					num = 4 - wilds;
					holes = 2;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH4_2K1:
					num = 4 - wilds;
					holes = 2;
					needed = 1;
					highcard = KING;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH4_3:
					num = 4 - wilds;
					holes = 3;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH3J1:
					needed = 1;
					highcard = JACK;
				case STRATEGY_STRFLUSH3:
					num = 3 - wilds;
					holes = 0;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH3_1J1:
					needed = 1;
					highcard = JACK;
					num = 3 - wilds;
					holes = 1;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH3_1J2:
					needed = 2;
					highcard = JACK;
				case STRATEGY_STRFLUSH3_1:
					num = 3 - wilds;
					holes = 1;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH3_2K1:
					needed = 1;
					highcard = KING;
					num = 3 - wilds;
					holes = 2;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH3_2J1:
					needed = 1;
					highcard = JACK;
					num = 3 - wilds;
					holes = 2;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH3_2J2:
					needed = 2;
					highcard = JACK;
				case STRATEGY_STRFLUSH3_2:
					num = 3 - wilds;
					holes = 2;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH3_3:
					num = 3 - wilds;
					holes = 3;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH2:
					num = 2 - wilds;
					holes = 0;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH2_1:
					num = 2 - wilds;
					holes = 1;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH2_2:
					num = 2 - wilds;
					holes = 2;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_STRFLUSH2_3:
					num = 2 - wilds;
					holes = 3;
					if (STRFLUSH(i, last, num, needed2, needed, index, n, low, hold, wildhold, hand.info, j, holes, highcard, type)) return true;
					break;
				case STRATEGY_FLUSH4:
					num = 4 - wilds;
					needed = 0;
					if (FLUSH(i, last, num, hand.info,needed2, needed, j, highcard, index, type, wildhold)) return true;
					break;
				case STRATEGY_FLUSH4_K1:
					num = 4 - wilds;
					needed = 1;
					highcard = KING;
					if (FLUSH(i, last, num, hand.info,needed2, needed, j, highcard, index, type, wildhold)) return true;
					break;
				case STRATEGY_FLUSH3_AK:
					num = 3 - wilds;
					needed = 2;
					highcard = KING;
					if (FLUSH(i, last, num, hand.info,needed2, needed, j, highcard, index, type, wildhold)) return true;
					break;
				case STRATEGY_FLUSH3_K1:
					num = 3 - wilds;
					needed = 1;
					highcard = KING;
					if (FLUSH(i, last, num, hand.info,needed2, needed, j, highcard, index, type, wildhold)) return true;
					break;
				case STRATEGY_FLUSH3_J2:
					num = 3 - wilds;
					needed = 2;
					highcard = JACK;
					if (FLUSH(i, last, num, hand.info,needed2, needed, j, highcard, index, type, wildhold)) return true;
					break;
				case STRATEGY_FLUSH3_J1:
					num = 3 - wilds;
					needed = 1;
					highcard = JACK;
					if (FLUSH(i, last, num, hand.info,needed2, needed, j, highcard, index, type, wildhold)) return true;
					break;
				case STRATEGY_FLUSH3:
					num = 3 - wilds;
					needed = 0;
					if (FLUSH(i, last, num, hand.info,needed2, needed, j, highcard, index, type, wildhold)) return true;
					break;

				case STRATEGY_STRAIGHT4J1:
					needed = 1;
					highcard = JACK;
				case STRATEGY_STRAIGHT4:
					num = 4 - wilds;
					holes = 0;
					if (STRAIGHT(i, j,last, num, needed2, needed, index, n, low, hand.info,next, holes, hold, wildhold, highcard,type)) return true;
					break;
				case STRATEGY_STRAIGHT3:
					num = 3 - wilds;
					holes = 0;
					if (STRAIGHT(i, j,last, num, needed2, needed, index, n, low, hand.info,next, holes, hold, wildhold, highcard,type)) return true;
					break;
				case STRATEGY_STRAIGHT2:
					num = 2 - wilds;
					holes = 0;
					if (STRAIGHT(i, j,last, num, needed2, needed, index, n, low, hand.info,next, holes, hold, wildhold, highcard,type)) return true;
					break;
				case STRATEGY_STRAIGHT4_1J3:
					needed = 3;
					highcard = JACK;
					num = 4 - wilds;
					holes = 1;
					if (STRAIGHT(i, j,last, num, needed2, needed, index, n, low, hand.info,next, holes, hold, wildhold, highcard,type)) return true;
					break;
				case STRATEGY_STRAIGHT4_1J2:
					needed = 2;
					highcard = JACK;
					num = 4 - wilds;
					holes = 1;
					if (STRAIGHT(i, j,last, num, needed2, needed, index, n, low, hand.info,next, holes, hold, wildhold, highcard,type)) return true;
					break;
				case STRATEGY_STRAIGHT4_1J1:
					needed = 1;
					highcard = JACK;
					num = 4 - wilds;
					holes = 1;
					if (STRAIGHT(i, j,last, num, needed2, needed, index, n, low, hand.info,next, holes, hold, wildhold, highcard,type)) return true;
					break;
				case STRATEGY_STRAIGHT4_1AK:
					needed = 2;
					highcard = KING;
				case STRATEGY_STRAIGHT4_1:
					num = 4 - wilds;
					holes = 1;
					if (STRAIGHT(i, j,last, num, needed2, needed, index, n, low, hand.info,next, holes, hold, wildhold, highcard,type)) return true;
					break;

				case STRATEGY_THREE:					// needed if 3 of a kind taken over a fullhouse
					num = 3;
					hold = 0;
					for (i = 0; i <= last - num + 1; i++)
					{
						if ((tStrategy[index].spotFlags & (1 << hand.info[i].spot)) == 0)
							continue;					// make sure this is a legal three of a kind
						n = num - 1;
						hold = (1 << hand.info[i].card);
						for (j = i + 1; j <= last; j++)
						{
							if (hand.info[j].spot != hand.info[i].spot)	
								continue;
							hold |= (1 << hand.info[j].card);
							n -= 1;						// one less card to get
							if (n == 0)					// we have 3 of a kind
							{
								nStrategyLine = index;
								nStrategy = type;
								nHold = hold;
								nSpot = hand.info[i].spot;	// override the value set by GetPayoff()
	//MYPRINTF((sDebug, "Three strategy for spot=%d hold=0x%x", info[i].spot, hold));
								return true;
							}
						} 
					}
					break;

				case STRATEGY_PAIR:
					if (hand.nWilds > 0)						// case with Joker + K for example
					{
						for (i = 0; i <= last; i++)
						{
							if ((tStrategy[index].spotFlags & (1 << hand.info[i].spot)) != 0)
							{
								nHold |= hand.info[i].nSpotHold;
								nStrategyLine = index;
								nStrategy = type;
	//MYPRINTF((sDebug, "Pair strategy for hold=0x%x", info[i].nSpotHold));
								return true;
							}
						}
					}
				case STRATEGY_TWOPAIR:
					if (hand.nPair >= 0
					&& (tStrategy[index].spotFlags & (1 << hand.info[hand.nPair].spot)) != 0)
					{
						nHold = hand.info[hand.nPair].nSpotHold;
						if (type == STRATEGY_TWOPAIR)
						{
							for (i = 0; i <= last; i++)
							{
								if (hand.info[i].nSpots == 2	// find second pair different from the first
								&&  hand.info[hand.nPair].spot != hand.info[i].spot)
									break;
							}
							if (i > last)
								break;
							nHold |= hand.info[i].nSpotHold;
						}
						nStrategyLine = index;
						nStrategy = type;
	//MYPRINTF((sDebug, "Pair strategy for npair=%d hold=0x%x", nPair, info[nPair].nSpotHold));
						return true;
					}
					break;

				case STRATEGY_HIGHCARD2:
					num = 2;
					for (i = 0; i <= last - num + 1; i++)
					{
						if ((tStrategy[index].spotFlags & (1 << hand.info[i].spot)) != 0)
						{
							num -= 1;
							hold |= 1 << hand.info[i].card;
							if (num == 0)				// we have our required number of cards
							{
								nStrategyLine = index;
								nStrategy = type;
								nHold = hold;
	//MYPRINTF((sDebug, "highcard strategy hold=0x%x", hold));
								return true;
							}
						}
					}
					break;
				case STRATEGY_HIGHCARD:
					num = 1;
					hold = wildhold;
					for (i = 0; i <= last - num + 1; i++)
					{
						if ((tStrategy[index].spotFlags & (1 << hand.info[i].spot)) != 0)
						{
							num -= 1;
							hold |= 1 << hand.info[i].card;
							if (num == 0)				// we have our required number of cards
							{
								nStrategyLine = index;
								nStrategy = type;
								nHold = hold;
	//MYPRINTF((sDebug, "highcard strategy hold=0x%x", hold));
								return true;
							}
						}
					}
					break;

				case PAYOUT_NONE:
					nStrategyLine = index;
					nStrategy = type;
					nHold = 0;
					return true;
				}
			}
			if (oneshot)
				return false;
			
			if (CONTINUE(index,hand.nWilds,hand.nWildHold))  {
				return true;
			} else {
				index++;
			}
			
		}
		// no entry given
		//ERRPRINTF((sDebug, "**** ERROR: end of strategy table reached"));
		nStrategyLine = index - 1;		// use last entry 
		nStrategy = PAYOUT_NONE;
		return true;
	}
	
	private static boolean ROYAL(int i, int j, int last, int num, Info[] info, int index, int type, int wildhold) {
		for (i = 0; i <= last - num + 1; i++)
		{
			if (info[i].nRoyal == num		
			&& (tStrategy[index].spotFlags & (1 << info[i].spot)) != 0)	// we can start with this spot
			{
				if (type == STRATEGY_ROYAL2AK
				||  type == STRATEGY_ROYAL3AK)
				{
					if ((info[i].nRoyalSpots & (BIT(ACE) | BIT(KING))) != (BIT(ACE) | BIT(KING)))
						break;			// not AK suited
				}
				else if (type == STRATEGY_ROYAL2T)
				{
//	MYPRINTF((sDebug, "RF2wT 0x%x", info[i].nRoyalSpots));
					if ((info[i].nRoyalSpots & BIT(TEN)) == 0)
						break;			// no T 
				}
				else if (type == STRATEGY_ROYAL2NOT)
				{
					if ((info[i].nRoyalSpots & BIT(TEN)) != 0)
						break;			// no T 
				}
				else if (type == STRATEGY_ROYAL2DK)
				{
					for (j = 0; j <= last; j++)
					{
						if (info[j].suit == info[i].suit)
						{
							if (info[j].spot < TEN)	// not part of our royal
								return false;
						}
						else
						{
							if (info[j].spot > EIGHT)	// another high card
								return false;
						}
					}
				}
				nStrategyLine = index;
				nStrategy = type;
				nHold = wildhold | info[i].nRoyalHold;
//	MYPRINTF((sDebug, "royal strategy=%d hold=0x%x royalHold=0x%x", type, nHold, info[i].nRoyalHold));
				return true;
			}
		}
		return false;
	}
	
	private static boolean NATSF(int i, int last, int num, int needed2, int needed, Info[] info, int index, int nWildSuits,int holes, int low, int n, int highcard, int type, int wildhold,int hold, int j) {
//			 NOTE this assumes the wild cards are still held!!
		//		so wild + 3456 will include the wild card
		for (i = 0; i <= last - num + 2; i++)	// look one card further to allow an ACELOW
		{
			needed2 = needed;
			if ((tStrategy[index].spotFlags & (1 << info[i].spot)) == 0)
				continue;						// we cannot start with this spot
			if (info[i].nSuits < num
			|| (nWildSuits & (1 << info[i].suit)) == 0)	// we must have the correct suited 2
				continue;
			n = num - 1;						// number more to get 
			if (deck.wildspot > info[i].spot)
			{
				holes -= deck.wildspot - info[i].spot - 1;
				if (holes < 0)					// no possible natural straight flush 
					break;
				low = deck.wildspot - n - holes - 1;
			}
			else if (type == STRATEGY_NATSF3x)	// holes are not valid for this strategy
			{
				low = info[i].spot - n;	
			}
			else								// wild card is less than the top card
			{
				low = info[i].spot - n - holes - 1;	// lowest number allowed including the wild card
				if (low > deck.wildspot)	// make sure we include the wild card
					continue;
			}
			hold = wildhold | (1 << info[i].card);
			if (info[i].spot >= highcard)		// high card
				needed2 -= 1;
			for (j = i + 1; j <= last && n > 0; j++)
			{
				if (info[j].suit == info[i].suit	// same suit
				&&  info[j].spot >= low)	// within range
				{
					n -= 1;						// one less card to get
					hold |= (1 << info[j].card);
					if (info[j].spot >= highcard)
						needed2 -= 1;
				}
			} 
			// check to see if ACELOW straights are allowed and we have an ACE of the right suit
			if (low <= ACELOW					// acelow is possible straight flush
			&&	(tStrategy[index].spotFlags & (1 << ACELOW)) != 0)
			{
				for (j = 0; j < last; j++)		// look for an ace suit match
				{
					if (info[j].spot != ACE)	// no more aces
						break;
					if (info[j].suit == info[i].suit)	// matching suit
					{
						n -= 1;							// one less card needed 
						hold |= (1 << info[j].card);	// hold the ace
						needed2 -= 1;					// an ace counts high too
						break;
					}
				}
			}
			if (n == 0)
			{
				if (needed2 > 0)			// we did not get the needed number of face cards
					break;
				nStrategyLine = index;
				nStrategy = type;
				nHold = hold;
//	MYPRINTF((sDebug, "straight flush strategy=%d hold=0x%x", type, nHold));
				return true;
			}
			return false;
		}
		return false;
	}
	
	private static boolean STRFLUSH(int i, int last, int num, int needed2, int needed, int index, int n, int low, int hold, int wildhold, Info[] info, int j, int holes, int highcard, int type) {
			for (i = 0; i <= last - num + 2; i++)	// look one card further to allow an ACELOW
			{
				needed2 = needed;
				if ((tStrategy[index].spotFlags & (1 << info[i].spot)) == 0)
					continue;					// we cannot start with this spot				
				n = num - 1;					// number more to get 
				low = info[i].spot - n - holes;	// lowest number allowed
				hold = wildhold | (1 << info[i].card);
				if (info[i].spot >= highcard)		// high card
					needed2 -= 1;
				for (j = i + 1; j <= last && n > 0; j++)
				{
					if (info[j].suit == info[i].suit	// same suit
					&&  info[j].spot >= low)	// within range
					{
						n -= 1;					// one less card to get
						hold |= (1 << info[j].card);
						if (info[j].spot >= highcard)
							needed2 -= 1;
					}
				} 
				// check to see if ACELOW straights are allowed and we have an ACE
				if (low <= ACELOW					// acelow is possible straight flush
				&&	(tStrategy[index].spotFlags & (1 << ACELOW)) != 0)
				{
					for (j = 0; j < last; j++)		// look for an ace suit match
					{
						if (info[j].spot != ACE)	// no more aces
							break;
						if (info[j].suit == info[i].suit)	// matching suit
						{
							n -= 1;							// one less card needed 
							hold |= (1 << info[j].card);	// hold the ace
							needed2 -= 1;					// an ace counts high too
							break;
						}
					}
				}
				if (n == 0)
				{
					if (needed2 > 0)			// we did not get the needed number of face cards
						break;
					nStrategyLine = index;
					nStrategy = type;
					nHold = hold;
//	MYPRINTF((sDebug, "straight flush strategy=%d hold=0x%x", type, nHold));
					return true;
				}
			}
			return false;
	}
	
	private static boolean FLUSH(int i, int last, int num, Info[] info,int needed2, int needed, int j, int highcard, int index, int type, int wildhold) {
		for (i = 0; i <= last - num + 1; i++)
		{
			if (info[i].nSuits == num)
			{
				// look for high cards needed
				needed2 = needed;
				for (j = i; j <= last && needed2 > 0; j++)
				{
					if (info[j].spot >= highcard
					&&  info[j].suit == info[i].suit)
						needed2 -= 1;
				}
				if (needed2 == 0)			// we got the high cards we wanted
				{
					nStrategyLine = index;
					nStrategy = type;
					nHold = wildhold | info[i].nSuitHold;
					return true;
				}
			}
		}
		return false;
	}
	
	private static boolean STRAIGHT(int i, int j,int last, int num, int needed2, int needed, int index, int n, int low, Info[] info,int next, int holes, int hold, int wildhold, int highcard,int type) {
		for (i = 0; i <= last - num + 1; i++)
		{
			needed2 = needed;
			if ((tStrategy[index].spotFlags & (1 << info[i].spot)) == 0)
				continue;						// we cannot start with this spot				
			n = num - 1;						// number more to get 
			low = info[i].spot - n - holes;	// lowest number allowed
			next = info[i].spot - 1;		// next card to get
			hold = wildhold | (1 << info[i].card);
			if (info[i].spot>= highcard)
				needed2 -= 1;
			for (j = i + 1; j <= last && n > 0; j++)
			{
				if (info[j].spot < low)	// not within range
					break;
				if (info[j].spot <= next)	// ignore duplicates
				{
					n -= 1;						// one less card to get
					next = info[j].spot - 1;
					hold |= (1 << info[j].card);
					if (info[j].spot >= highcard)
						needed2 -= 1;
				}
			} 
			// check to see if ACELOW straights are allowed and we have an ACE
			if (low <= ACELOW					// acelow is possible straight
			&&  info[0].spot == ACE		// we have an ACE and ACELOW is allowed
			&&	(tStrategy[index].spotFlags & (1 << ACELOW)) != 0)
			{
				n -= 1;							// one less card needed 
				hold |= (1 << info[j].card);	// hold the ace
				needed2 -= 1;					// an ace counts high too
			}
			if (n == 0)
			{
				if (needed2 > 0)				// we did not get the needed number of face cards
					break;
				nStrategyLine = index;
				nStrategy = type;
				nHold = hold;
				return true;
			}
		}
		return false;
	}
	
	private static boolean CONTINUE(int index,int nWilds,int nWildHold) {
		index++;
		if ((byte)tStrategy[index].wilds != nWilds)
		{
			index -= 1;
			nStrategyLine = index;
			nStrategy = tStrategy[index].type;	// use last entry
			nHold = nWildHold;
//				//MYPRINTF(("End of list at %d using wild cards only", index));
			return true; 			
		}
		return false;
	}
		
	/*
	 * ------------------ RMS Load and Save Functions ----------------------------------------
	 * 
	 * Meathods used to maintain the persistence of game States and Statistics
	 */
	
	
	
	public static void loadGameSettings() {
			
		try {
			
		    RecordStore load = RecordStore.openRecordStore(Strings.RECORD_SETTINGS,true);
			
			if (load.getNumRecords() != 0) {
				
				// Load Game Settings
				ByteArrayInputStream iStream = new ByteArrayInputStream(load.getRecord(1));
				DataInputStream reader = new DataInputStream(iStream);
				avSetting = reader.readByte();
				hSetting = reader.readByte();
				VPHandler.selectedGame = reader.readByte();
				for (int c = 0; c < NUM_GAMES; c++) {
					gamePlayed[c] = reader.readBoolean();
					System.out.println(GAME_LIST[c] + ": " + gamePlayed[c]);
				}
				load.closeRecordStore();
				iStream.close();
				reader.close();				
				//#ifdef DEBUG
				System.out.println("Settings: Load: Successful");
				//#endif
			} else {
				//#ifdef DEBUG
				System.out.println("Settings: Load: No Record");
				//#endif
			}
		} catch (Exception e) {
			//System.out.println("Settings: Load: "+ e);
		}
		System.gc();
		//System.out.println("                : " + (rntm.totalMemory() - rntm.freeMemory()));
	}
	
	public static void createNew() {
		//#ifdef DEBUG
	    System.out.print("Creating new: ");
	    //#endif
	    // First set our default values
	    nWins = new int[tPayoffSize];
		
		nCoins = 500;
		gState = Game.STATE_PREGAME;
		nSelect = pokerScreen.CURSOR_BET;
		nPayoff = NONE;
		nStrategyLine = NONE;
		nSpot = NONE;
		nBet = 5;	
		nSelect = NONE;  // None == cursorBet
		
		nCoinsWon=0;				// number of coins won or lost (starts at 0)
		nCoinsBet=0;				// amount wagered
		nHands=0;					// hands played
		nHandsWon=0;				// hands we won 
		nErrors=0;				// errors made

		nSeconds=0;				// time played (seconds)
		nLastWin=0;				// current coins won (0=none)			
		nSelect=0;				// what button is selected (see SELECT_xxx)
		nStrategyLine=0;				// which strategy we are using use (index into tStrategy[])
		
		gState = STATE_PREGAME;
		
		nHold = 0;
	    
		try {
			
			RecordStore load;
			if (VPHandler.selectedGame == TOURNAMENT) {
				try {
					RecordStore.deleteRecordStore(Strings.RECORD_TOURNAMENT);
				} catch (Exception e) {
					// Not a problem
				}
				load = RecordStore.openRecordStore(Strings.RECORD_TOURNAMENT,true);	
			} else {
				Game.gamePlayed[activeGame] = false;
				try {
					RecordStore.deleteRecordStore(Strings.RECORD_HEADER + gStrings[0][0]);
				} catch (Exception e) {
					// Not a problem
				}
				load = RecordStore.openRecordStore(Strings.RECORD_HEADER + gStrings[0][0],true);
			}
			
			byte[] out = byteEncodeGame();
			load.addRecord(out,0,out.length);	
			
			load.closeRecordStore();
			//#ifdef DEBUG
			System.out.println("Successful");
			//#endif
		} catch (Exception e) {
			//#ifdef DEBUG
		    System.out.println("FAILED");
			System.out.println("GameCreate: "+ e);
			//#endif
		}
		GameScreen.timeStopPlay = 0;
	}
	
	public static void saveGame() {
	    
	    try {

	    	RecordStore save;
	    	
		    if (curGameType != NONE) {
				
		    	//#ifdef DEBUG
		    	// Save Game Information
			    System.out.print("saveGame: Saving Game: " + gStrings[0][0]);
				//#endif
	    	
			    // Save Game Information
				if (curGameType == TOURNAMENT) {
					save = RecordStore.openRecordStore(Strings.RECORD_TOURNAMENT,true);
				} else {
					save = RecordStore.openRecordStore(Strings.RECORD_HEADER + gStrings[0][0],false);
				}
				
				byte[] out = byteEncodeGame();
				save.setRecord(1,out,0,out.length);
				save.closeRecordStore();
				
				//#ifdef DEBUG
				System.out.println(" SUCCESSFUL");
				//#endif
		    } 
		    
			// Save Setting Information
		    try {
		    	RecordStore.deleteRecordStore(Strings.RECORD_SETTINGS);
		    } catch (Exception e) {
		    	// not important
		    }
			save = RecordStore.openRecordStore(Strings.RECORD_SETTINGS,true);
			ByteArrayOutputStream output = new ByteArrayOutputStream();
			DataOutputStream writer = new DataOutputStream(output);
			writer.writeByte(avSetting); 
			writer.writeByte(hSetting);
			writer.writeByte(activeGame);
			for (int c = 0; c < NUM_GAMES; c++) writer.writeBoolean(gamePlayed[c]);
			save.addRecord(output.toByteArray(),0,output.size());
			save.closeRecordStore();
			output.close();
			writer.close();
			// System.out.println("Settings: Save: Successful");
		    
		} catch (Exception e) {
			//#ifdef DEBUG
		    System.out.println(" FAILED: saveGame: " + e);
			e.printStackTrace();
			//#endif
		}	
	}
	
	public static byte[] byteEncodeGame() {
	    
	    try {
	        
		    ByteArrayOutputStream output = new ByteArrayOutputStream();
			DataOutputStream writer = new DataOutputStream(output);
			
			if (curGameType == TOURNAMENT) {
				writer.writeInt(TournInfo.nGame);
			} else {
				writer.writeInt(NONE);
			}
			
			writer.writeInt(nCoins);					// number of coins (starts at START_COINS)
			writer.writeInt(nCoinsWon);				// number of coins won or lost (starts at 0)
			writer.writeInt(nCoinsBet);				// amount wagered
			writer.writeInt(nSeconds);				// time played (seconds)
			writer.writeInt(nHold);
			
			writer.writeShort(nHands);					// hands played
			writer.writeShort(nHandsWon);				// hands we won 
			writer.writeShort(nErrors);				// errors made
			
			writer.writeByte(gState);
			writer.writeByte(nLastWin);				// current coins won (0=none)	
			writer.writeByte(nSelect);				// what button is selected (see SELECT_xxx)
			writer.writeByte(nPayoff);				// index o payoff table
			writer.writeByte(nStrategyLine);			// which strategy we are using use (index o pStrategyTable[])
			writer.writeByte(nSpot);
			writer.writeByte(nBet);					// amount we are betting
			
			int c;
			
			for (c = 0; c < hand.info.length; c++) {
				writer.writeByte(hand.info[c].suit);
				writer.writeByte(hand.info[c].spot);
			}
			for (c = 0; c < deck.cards.length; c++) {
				writer.writeByte(deck.cards[c].suit);
				writer.writeByte(deck.cards[c].spot);
			}
			for (c = 0; c < nWins.length; c++) {
				writer.writeShort(nWins[c]);	// wins for each type
			}
			
			return output.toByteArray();
			
	    } catch (Exception e) {
	        //#ifdef DEBUG
		    System.out.println("FAILED");
			System.out.println("byteEncodeGame: "+ e);
			e.printStackTrace();
			//#endif
		}
	    
	    return null;
	}
}